<?php

namespace App\Http\Controllers;

use App\Models\Orders;
use App\Models\Record;
use App\Models\Setting;
use App\Models\StripeCustomer;
use App\Models\Subscription;
use App\Models\User;
use Hash;
use Illuminate\Http\Request;
use Validator;
use Auth;

class StripeController extends Controller
{
    public function purchase_subscription(Request $request)
    {
        $subscription = Subscription::findOrFail($request->subscriptionid);
        try {
            $sk = Setting::first()->stripe_secret;
            $stripe = new \Stripe\StripeClient($sk);
            $customer = $stripe->customers->create([
                'description' => "Purchase " . $subscription->heading . " Subscription",
            ]);

            $order = Orders::create([
                'userid' => Auth::user()->id,
                'subscriptionid' => $subscription->id,
                'heading' => $subscription->heading,
                'price' => $subscription->price,
                'records' => $subscription->records,
            ]);

            StripeCustomer::create([
                'user_id' => Auth::user()->id,
                'subscription_id' => $subscription->id,
                'customer_id' => $customer->id,
                'orderid' => $order->id,
            ]);

            $session = $stripe->checkout->sessions->create([
                'customer' => $customer,
                'line_items' => [
                    [
                        'price_data' => [
                            'currency' => 'usd',
                            'product_data' => [
                                'name' => 'Purchase ' . $subscription->heading . ' Subscription'
                            ],
                            'unit_amount' => $subscription->price * 100
                        ],
                        'quantity' => 1,
                    ],
                ],
                'mode' => 'payment',
                'success_url' => url('/successpayment?session_id={CHECKOUT_SESSION_ID}&order_id=' . encrypt($order->id)),
                'cancel_url' => route('subscription'),
            ]);
            return redirect()->away($session->url);
        } catch (Exception $e) {
            return back()->withError($e->getMessage());
        }
    }

    public function successpayment(Request $request)
    {
        $sessionid = $request->session_id;
        $orderid = $request->order_id;

        $secret_key = Setting::first()->stripe_secret;
        $stripe = new \Stripe\StripeClient($secret_key);
        $session = $stripe->checkout->sessions->retrieve(
            $sessionid
        );
        if($session->payment_status == 'paid'){
            $existing = Orders::findOrFail(decrypt($orderid));
            $existing->update([
                'payment_status' => 'paid'
            ]);
        }
        return redirect()->route('record')->with('success','Successfully Purchased.');
    }

    public function customsuccesspayment(Request $request)
    {
        $sessionid = $request->session_id;
        $orderid = $request->order_id;

        $secret_key = Setting::first()->stripe_secret;
        $stripe = new \Stripe\StripeClient($secret_key);
        $session = $stripe->checkout->sessions->retrieve(
            $sessionid
        );
        if($session->payment_status == 'paid'){
            $existing = Record::findOrFail(decrypt($orderid));
            $existing->update([
                'payment_status' => 'paid',
                'status' => '1'
            ]);
        }
        return redirect()->route('manage_records_list')->with('success', 'Record Saved Successfully.');
    }

}
