<?php

namespace App\Http\Controllers;

use App\Mail\AccountRegistration;
use App\Mail\PasswordResetMail;
use App\Models\Contactus;
use App\Models\Orders;
use App\Models\Record;
use App\Models\Setting;
use App\Models\User;
use App\Models\UsersLoginHistory;
use App\Models\Wishes;
use Auth;
use Carbon\Carbon;
use Hash;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Clipboard;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Stripe\Customer;
use Stripe\Stripe;
use Stripe\Subscription;

class UsersController extends Controller
{
    public function userlist()
    {
        $users = User::where('type', 'user')
            ->orderBy('id', 'desc')->get();
        return view("admin.userlist", compact('users'));
    }


    public function userdetail($id)
    {
        $user = User::findOrFail(decrypt($id));
        return view("admin.users.userdetail", compact('user'));
    }

    public function AddUsers()
    {
        return view("admin.users.addusers");
    }

    public function saveuserrecord(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|unique:users,email',
            'password' => 'required|min:8',
            'address' => 'required',
            'email_verification' => 'required',
            'status' => 'required'
        ]);
        if ($validator->fails()) {
            return back()->withErrors($validator);
        }
        $refral_code = rand(1000, 1000000);
        $data['refral_code'] = $refral_code;
        $data['name'] = $request->name;
        $data['email'] = $request->email;
        $data['password'] = Hash::make($request->password);
        $data['email_verification'] = $request->email_verification;
        $data['status'] = $request->status;
        $data['address'] = $request->address;
        $data['register_by'] = "admin";

        if ($request->hasFile('image')) {
            $image = $request->file('image');
            $image_new = time() . $image->getClientOriginalName();
            $image->move('admin/assets/uploads/', $image_new);
            $data['image'] = $image_new;
        }
        user::create($data);
        return redirect()->route('admin.UsersList')->with('success', 'Data Successfully Added !');
    }

    public function deleteuser($id)
    {
        $getData = user::findOrFail(decrypt($id))->delete();
        return back()->with('success', 'Data Deleted Successfully !');
    }

    public function update_user(Request $request)
    {
        $EmailCheck = user::where('email', $request->email)->where('id', '!=', $request->id)->count();
        if ($EmailCheck > 0) {
            return back()->with('error', 'This email is already taken.');
            dd("stop");
        }

        $GetData = User::findOrFail($request->id);
        $GetData->name = $request->name;
        $GetData->phone = $request->phone;
        $GetData->email = $request->email;
        $GetData->email_verification = $request->email_verification;
        $GetData->status = $request->status;
        if (isset($request->password)) {

            $validator = Validator::make($request->all(), [
                'password' => 'required|string|min:8'
            ]);

            if ($validator->fails()) {
                return back()->with('error', 'Please Enter Valid Password !');
            }
            $GetData->password = Hash::make($request->password);
        }

        $GetData->save();
        return back()->with('success', 'Profile Updated Successfully !');
    }

    public function deletecontactmessage($id)
    {
        $getData = Contactus::findOrFail(decrypt($id));
        $getData->delete();
        return back()->with('success', 'Data Deleted Successfully !');
    }

    public function edituser($id)
    {
        $user = user::findOrFail(decrypt($id));
        return view('admin.users.edituser', compact('user'));
    }

    public function userdetailshistory($id)
    {
        $user = user::findOrFail(decrypt($id));
        return view('admin.users.userdetailshistory', compact('user'));
    }

    public function updateuserrecord(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'name' => 'required',
            'email' => 'required',
            'address' => 'required',
            'email_verification' => 'required',
            'status' => 'required'
        ]);
        if ($validator->fails()) {
            return back()->withErrors($validator);
        }

        $getData = user::findOrFail(decrypt($request->id));

        $EmailCheck = user::where('email', $request->email)->where('id', '!=', $getData->id)->count();
        if ($EmailCheck > 0) {
            return back()->with('error', 'This email is already taken.');
            dd("stop");
        }

        if (isset($request->name)) {
            $getData->name = $request->name;
        }
        if (isset($request->email)) {
            $getData->email = $request->email;
        }
        if (isset($request->password)) {
            $validator = Validator::make($request->all(), [
                'password' => 'required|min:8'
            ]);
            if ($validator->fails()) {
                return back()->withErrors($validator);
            }
            $getData->password = Hash::make($request->password);
        }

        if (isset($request->status)) {
            $getData->status = $request->status;
        }
        if (isset($request->email_verification)) {
            $getData->email_verification = $request->email_verification;
        }
        if (isset($request->address)) {
            $getData->address = $request->address;
        }

        if ($request->hasFile('image')) {

            $validator = Validator::make($request->all(), [
                'image' => 'required|image|mimes:jpg,png,jpeg,gif,svg'
            ]);

            if ($validator->fails()) {
                return back()->with('error', 'Invalid Image');
            }

            if (\File::exists(public_path('admin/assets/uploads/' . $getData->image))) {

                \File::delete(public_path('admin/assets/uploads/' . $getData->image));
            }


            $image = $request->file('image');
            $image_new = time() . $image->getClientOriginalName();
            $image->move('admin/assets/uploads/', $image_new);
            $getData->image = $image_new;
        }
        $getData->save();
        return redirect()->route('admin.UsersList')->with('success', 'Data Successfully Added !');
    }

    public function UsersLoginHistory()
    {
        $usersloginhistory = UsersLoginHistory::orderBy('id', 'desc')->get();
        return view("admin.users.UsersLoginHistory", compact('usersloginhistory'));
    }


//    website user code
    public function UserRegister()
    {
        $setting = Setting::first();
        if (Auth::Check() && Auth::user()->type == "user") {
            return redirect()->route('/');

        } else {
            return view("website.users.userregister", compact('setting'));
        }
    }

//    saveuser
    public function saveuser(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'name' => 'required',
                'phone' => 'required',
                'email' => 'required|unique:users,email',
                'password' => 'min:8|required_with:password_confirmation|same:password_confirmation',
                'password_confirmation' => 'min:8'
            ]);
            if ($validator->fails()) {
                return back()
                    ->withInput()
                    ->withErrors($validator);
            }
            $data['name'] = $request->name;
            $data['email'] = $request->email;
            $data['phone'] = $request->phone;
            $data['password'] = \Illuminate\Support\Facades\Hash::make($request->password);
            $userdata = user::create($data);
            $maildata = $userdata;
            $setting = Setting::first();
            $websiteUrl = url('/admin/assets/uploads/' . $setting->logo);
            $maildata->logo = $websiteUrl;
            Mail::to($request->email)->send(new AccountRegistration($maildata));
            return redirect()->route('login')->with('success', 'Successfully Registered ! Please Check your email.');

        } catch (\Exception $e) {
            return back()
                ->withInput()
                ->withErrors($e->getMessage());
        }
    }

    public
    function verifyaccount($id)
    {
        $id = decrypt($id);
        $user = User::findOrFail($id);
        if ($user->email_verification == "0") {
            $user->email_verification = "1";
            $user->status = "active";
            $user->email_verified_at = Carbon::now();
            $user->save();
            return redirect()->route('login')->with('success', 'Account Verified Successfully');
        } else {
            return redirect()->route('login')->with('error', 'Already Verified.');
        }

    }


    public
    function ForgetPassword()
    {
        return view("website.users.forgetpassword");
    }

//    resetpassword
    public
    function resetpassword(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'email' => 'required'
        ]);
        if ($validator->fails()) {
            return back()
                ->withInput()
                ->withErrors($validator);
        }

        $check = User::where('email', $request->email)->first();
        $setting = Setting::first();
        if ($check) {
            if ($check->status == "active") {

                $check->remember_token = Str::random(30);
                $check->save();
                $setting = Setting::first();
                $maildata = $check;
                $maildata['logo'] = url('/admin/assets/uploads/' . $setting->logo);
                Mail::to($request->email)->send(new PasswordResetMail($maildata));
                return redirect()->route('login')->with('success', 'Password Sent ! Please Check your email address !');
            } else {
                return back()
                    ->withInput()
                    ->withErrors('Your account is not active please contact with customer support.');
            }
        } else {
            return back()
                ->withInput()
                ->with('error', 'Invalid email !Please enter valid email.')
                ->withErrors('Invalid email !Please enter valid email.');
        }

    }

//resetnewpassword
    public
    function resetnewpassword($email)
    {
        $decemail = decrypt($email);
        $check = User::where('email', $decemail)->whereNotNull('remember_token')->count();
        if ($check > 0) {
            return view('website.resetnewpasswordform', compact('email'));
        } else {
            return redirect()->route('login')->with('error', 'Link Expired Please Reset Your Password Again !');
        }
    }

    public
    function saveresetnewpassword(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'password' => 'min:8|required_with:password_confirmation|same:password_confirmation',
            'password_confirmation' => 'min:8'
        ]);
        if ($validator->fails()) {
            return back()
                ->withInput()
                ->withErrors($validator);
        }

        $email = decrypt($request->id);
        $user = User::where('email', $email)->first();
        $user->password = Hash::make($request->password);;
        $user->remember_token = NULL;
        $user->save();
        return redirect()->route('login')->with('success', 'Password Successfully Updated !');
    }

    public
    function UserLogin()
    {
        if (Auth::check() && Auth::user()->type == "user") {
            return redirect()->route('/');

        } else {
            return view("website.users.UserLogin");
        }
    }

    public
    function customerlogin(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'email' => 'required',
                'password' => 'required|min:8'
            ]);
            if ($validator->fails()) {
                return back()->withInput()->withErrors($validator);
            }
            if (auth()->attempt(['email' => $request->email, 'password' => $request->password, 'type' => 'user'])) {
                if (Auth::user()->type == "user" && Auth::user()->status == "inactive") {
                    Auth::logout();
                    return redirect()->back()->withInput()->with('error', 'Status Not Active ! Please Contact with service provider.')->withErrors("Status Not Active ! Please Contact with service provider.");
                } else {
                    if (Auth::user()->email_verification == "0") {
                        Auth::logout();
                        return redirect()->back()->withInput()->with('error', 'Email Not Verified ! Please Contact with service provider.')->withErrors("Email Not Verified ! Please Contact with service provider.");
                    } else {
                        if (Auth::check()) {
                            return redirect()->route('listing_agent');
                        }
                    }
                }


            } else {
                return redirect()->back()->withInput()->with('error', 'Invalid Email or Password !')->withErrors("Invalid Email or Password !");
            }

        } catch (\Exception $e) {
            return redirect()->route('login')
                ->withInput()
                ->withErrors($e->getMessage());
        }

    }

    public
    function customerlogout(Request $request)
    {
        Auth::logout();
        return redirect()->route("/");
    }

    public
    function userprofile(Request $request)
    {
        $setting = Setting::first();
        return view('website.users.userprofile', compact('setting'));
    }

    public
    function updateuserprofile(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'phone' => 'required'
        ]);
        if ($validator->fails()) {
            return back()->withErrors($validator);
        }

        $getData = user::findOrFail(Auth::user()->id);

        if (isset($request->name)) {
            $getData->name = $request->name;
        }
        if (isset($request->phone)) {
            $getData->phone = $request->phone;
        }

        if (isset($request->password)) {
            $validator = Validator::make($request->all(), [
                'password' => 'required|min:8'
            ]);
            if ($validator->fails()) {
                return redirect()->back()->with('error', 'Minimum 8 Characters Required for password.');
            }
            $getData->password = Hash::make($request->password);
        }
        $getData->save();
        return redirect()->back()->with('success', 'Data Successfully Added !');
    }

    public
    function upgradelevelpayment(Request $request)
    {
        $setting = Setting::first();
        $level = Levels::first();
        try {

            $validator = Validator::make($request->all(), [
                'id' => 'required',
                'card_name' => 'required|string|max:255',
                'card_number' => 'required|string|min:16',
                'card_expiry' => 'required|string|max:7',
                'card_cvc' => 'required|string|min:3|max:3',
                'stripeToken' => 'required|string',
            ]);
            if ($validator->fails()) {
                return back()
                    ->withInput()
                    ->withErrors($validator);
            }
            $leveldata = UserLevels::findOrFail(decrypt($request->id));

//            payment code
            $a = Stripe::setApiKey($setting->stripe_secret);


            //            create customer id
            $custome_id = "";
            if (empty(Auth::user()->customer_id) || Auth::user()->customer_id == "") {
                $customer = Customer::create([
                    'email' => Auth::user()->email,
                    'source' => $request->input('stripeToken'),
                ]);
                $checkuserdata = User::findOrFail(Auth::user()->id);
                $checkuserdata->customer_id = $customer->id;
                $checkuserdata->save();
                $custome_id = $customer->id;
            } else {
                $custome_id = Auth::user()->customer_id;
            }

            $firstTimeCharge = \Stripe\Charge::create([
                'amount' => $leveldata->price,
                'customer' => $custome_id,
                'currency' => 'jpy',
                'description' => 'Level Upgration',
            ]);

//            =================cancel and subscribe new subscription start =========

            $subscriptions = Subscription::all(['customer' => $custome_id]);
            if ($subscriptions) {
                foreach ($subscriptions as $subscription) {
                    if ($subscription->plan->id == Auth::user()->current_level->level_record->product_id) {
                        $subscription->cancel();
                    }
                }
            }
            $newsubscription = Subscription::create([
                'customer' => $custome_id,
                'items' => [
                    [
                        'price' => $leveldata->level_record->product_id,
                    ],
                ],
            ]);

//            =================cancel and subscribe new subscription end =========


            if ($firstTimeCharge->status == "succeeded") {

                UserLevels::where('userid', Auth::guard('web')->user()->id)->where('status', 'active')->update(['status' => 'completed']);
                $leveldata->status = "active";
                $leveldata->save();

                $paymentdata['userid'] = Auth::guard('web')->user()->id;
                $paymentdata['amount'] = ($firstTimeCharge->amount / 100);
                $paymentdata['receipt_url'] = $firstTimeCharge->receipt_url;
                $paymentdata['status'] = $firstTimeCharge->status;
                $paymentdata['type'] = $leveldata->heading . ' Upgration';
                UserPayment::create($paymentdata);

            }

            return redirect()->route('managelevels')->with('success', 'Successfully Paid & Level Updated !');

        } catch (\Exception $e) {
            return back()
                ->withInput()
                ->withErrors($e->getMessage());
        }
    }


    public function activesubscriptions()
    {
        $activesubscriptions = Orders::where('userid', Auth::user()->id)->where('status', 'active')->get();
        $setting = Setting::first();
        return view('website.users.activesubscriptions', compact('setting', 'activesubscriptions'));
    }

    public function completedsubscriptions()
    {
        $completedsubscriptions = Orders::where('userid', Auth::user()->id)->where('status', 'completed')->get();
        $setting = Setting::first();
        return view('website.users.completedsubscriptions', compact('setting', 'completedsubscriptions'));
    }

    public function edit_record($id)
    {
        $record = Record::findOrFail(decrypt($id));
        return view('admin.edit_record', compact('record'));
    }

    public function managerecord(Request $request)
    {
        $records = Record::all();
        return view('admin.managerecord', compact('records'));
    }

    public function delete_record($id)
    {
        $getData = Record::findOrFail(decrypt($id))->delete();
        return back()->with('success', 'Data Deleted Successfully !');
    }

    public function update_record_from_admin(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'name' => 'required',
            'phone' => 'required',
            'address' => 'required',
            'postal_code' => 'required',
            'state' => 'required',
            'city' => 'required',
            'mls_number' => 'required',
            'commission' => 'required',
            'type' => 'required'
        ]);

        if ($validator->fails()) {
            return back()->withInput()->with('error', 'All fields required !');
        }
        $rec = Record::findOrFail($request->id);
        $rec->name = $request->name;
        $rec->phone = $request->phone;
        $rec->address = $request->address;
        $rec->postal_code = $request->postal_code;
        $rec->state = $request->state;
        $rec->city = $request->city;
        $rec->mls_number = $request->mls_number;
        $rec->commission = $request->commission;
        $rec->type = $request->type;
        $rec->status = $request->status;
        $record = $rec->update();
        if ($record) {
            return redirect()->route('admin.managerecord')->with('success', 'Record Updated Successfully.');
        } else {
            return back()->withInput()->with('error', 'Record Not Updated.');
        }
    }


}
